use crate::domain::entity::{PokemonName, PokemonNumber, PokemonTypes};
use crate::repo::pokemon::{Insert, Repository};

struct Request {
    number: u16,
    name: String,
    types: Vec<String>,
}

enum Response {
    Ok(u16),
    BadRequest,
    Conflict,
}

fn execute(repo: &mut dyn Repository, req: Request) -> Response {
    println!("execute");
    match (
        PokemonNumber::try_from(req.number),
        PokemonName::try_from(req.name),
        PokemonTypes::try_from(req.types)
        ) {
        (Ok(number), Ok(name), Ok(types)) => match repo.insert(number, name, types){
            Insert::Ok(num) => Response::Ok(u16::from(num)),
            Insert::Conflict => Response::Conflict,
        },
        _ => Response::BadRequest,
    }
}

#[cfg(test)]
mod tests {
    use crate::repo::pokemon::InMemoryRepository;
    use super::*;

    #[test]
    fn it_should_return_the_pokemon_number_otherwise() {
        let mut repo = InMemoryRepository::new();
        let number = 25;
        let req = Request {
            number: number,
            name: "Pikachu".to_string(),
            types: vec!["Electric".to_string(), ],
        };
        let res = execute(&mut repo, req);
        match res {
            Response::Ok(num) => assert_eq!(num, number),
            _ => unreachable!(),
        }
    }

    #[test]
    fn it_should_return_a_bad_request_error_when_request_is_invalid() {
        let mut repo = InMemoryRepository::new();
        let number = 25;
        let req = Request {
            number: number,
            name: "".to_string(),
            types: vec!["Electric".to_string(), ],
        };
        let res = execute(&mut repo, req);
        match res {
            Response::Ok(num) => assert_eq!(num, number),
            Response::BadRequest => {},
            _ => unreachable!()
        }
    }

    #[test]
    fn it_should_return_a_conflict_error_when_pokemon_number_already_exists() {
        let number = PokemonNumber::try_from(25).unwrap();
        let name = PokemonName::try_from("Pikachu".to_string()).unwrap();
        let types = PokemonTypes::try_from(vec!["Electric".to_string(),]).unwrap();
        let mut repo = InMemoryRepository::new();
        repo.insert(number, name, types);
        let req = Request {
            number: 25,
            name: "Charmander".to_string(),
            types: vec!["File".to_string()],
        };
        let res = execute(&mut repo, req);
        match res {
            Response::Conflict => {},
            _ => unreachable!(),
        }
    }
}